@@ -1,15 +1,16 @@
-use crate::common::*;
+use crate::{common::*, tcp_pool::TcpPool};
 
 use crate::reqs::*;
 use async_net::TcpStream;
 
+use dashmap::DashMap;
 use lazy_static::lazy_static;
 
 use serde::{de::DeserializeOwned, Serialize};
 use smol::lock::Semaphore;
 
-use std::net::SocketAddr;
 use std::time::{Duration, Instant};
+use std::{net::SocketAddr, sync::Arc};
 
 lazy_static! {
     static ref CONN_POOL: Client = Client::default();
@@ -28,40 +29,10 @@ pub async fn request<TInput: Serialize + Clone, TOutput: DeserializeOwned + std:
 /// Implements a thread-safe pool of connections to melnet, or any HTTP/1.1-style keepalive protocol, servers.
 #[derive(Default)]
 pub struct Client {
-    // pool: DashMap<SocketAddr, deadpool::managed::Pool<TcpPoolManager>>,
+    pool: DashMap<SocketAddr, Arc<TcpPool>>,
 }
 
 impl Client {
-    // /// Connects to a given address, which may return either a new connection or an existing one.
-    // async fn connect(&self, addr: impl ToSocketAddrs) -> std::io::Result<Object<TcpPoolManager>> {
-    //     let addr = addr.to_socket_addrs()?.next().unwrap();
-    //     let existing = self
-    //         .pool
-    //         .entry(addr)
-    //         .or_insert_with(|| {
-    //             Pool::builder(TcpPoolManager(addr))
-    //                 .max_size(64)
-    //                 .build()
-    //                 .unwrap()
-    //         })
-    //         .clone();
-    //     loop {
-    //         let conn = existing.get().await.map_err(|err| match err {
-    //             PoolError::Timeout(_) => panic!("should never see deadpool timeout"),
-    //             PoolError::Closed => panic!("closed"),
-    //             PoolError::NoRuntimeSpecified => panic!("what"),
-    //             PoolError::Backend(err) => err,
-    //             PoolError::PostCreateHook(_) => todo!(),
-    //             PoolError::PreRecycleHook(_) => todo!(),
-    //             PoolError::PostRecycleHook(_) => todo!(),
-    //         })?;
-    //         if conn.expired() {
-    //             continue;
-    //         }
-    //         return Ok(conn);
-    //     }
-    // }
-
     /// Does a melnet request to any given endpoint.
     pub async fn request<TInput: Serialize + Clone, TOutput: DeserializeOwned + std::fmt::Debug>(
         &self,
@@ -99,7 +70,12 @@ impl Client {
         let _guard = GLOBAL_LIMIT.acquire().await;
         let start = Instant::now();
         // grab a connection
-        let mut conn = TcpStream::connect(addr)
+        let mut conn = self
+            .pool
+            .entry(addr)
+            .or_insert_with(|| TcpPool::new(32, Duration::from_secs(5), addr).into())
+            .clone()
+            .connect()
             .await
             .map_err(MelnetError::Network)?;
 
@@ -130,7 +106,13 @@ impl Client {
             };
             let elapsed = start.elapsed();
             if elapsed.as_secs_f64() > 3.0 {
-                log::warn!("melnet req to {} took {:?}", addr, elapsed)
+                log::warn!(
+                    "melnet req of verb {}/{} to {} took {:?}",
+                    netname,
+                    verb,
+                    addr,
+                    elapsed
+                )
             }
             Ok::<_, crate::MelnetError>(response)
         };
